@@ -53,8 +53,14 @@ def _exit():
 
 def get_parser() -> typing.ArgumentParser:
     """Argument parser."""
+    from darc import __version__  # pylint: disable=import-outside-toplevel
+
     parser = argparse.ArgumentParser('darc',
                                      description='the darkweb crawling swiss army knife')
+    parser.add_argument('-v', '--version', action='version', version=__version__)
+
+    parser.add_argument('-t', '--type', action='store', required=True,
+                        choices=['crawler', 'loader'], help='type of worker process')
 
     parser.add_argument('-f', '--file', action='append', help='read links from file')
     parser.add_argument('link', nargs=argparse.REMAINDER, help='links to craw')
@@ -92,7 +98,8 @@ def main():
                     link_list.append(line)
 
     # write to database
-    save_requests((parse_link(link) for link in link_list), score=0, nx=True)
+    link_pool = [parse_link(link) for link in link_list]
+    save_requests(link_pool, score=0, nx=True)
 
     if DEBUG:
         print(stem.util.term.format('-' * shutil.get_terminal_size().columns, stem.util.term.Color.MAGENTA))  # pylint: disable=no-member
@@ -103,7 +110,7 @@ def main():
             print('proxy,scheme,host,hash,link', file=file)
 
     try:
-        process()
+        process(args.type)
     except BaseException:
         traceback.print_exc()
     _exit()
